/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.text.Document;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.panels.support.TestRunComponentEnabler;
import com.eviware.soapui.impl.wsdl.panels.teststeps.support.GotoTestStepsComboBoxModel;
import com.eviware.soapui.impl.wsdl.submit.transports.http.WsdlResponse;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlGotoTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlGotoTestStep.GotoCondition;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.support.DocumentListenerAdapter;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JUndoableTextArea;
import com.eviware.soapui.support.xml.XmlUtils;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for WsdlGotoTestSteps
 * 
 * @author Ole.Matzura
 */

public class GotoStepDesktopPanel extends JPanel implements DesktopPanel
{
	private final WsdlGotoTestStep gotoStep;
	private DefaultListModel listModel;
	private JList conditionList;
	private JTextArea expressionArea;
	private JButton copyButton;
	private JButton deleteButton;
	private JButton declareButton;
	private GotoTestStepsComboBoxModel testStepsModel;
	private JComboBox testStepsCombo;
	private JButton testConditionButton;
	private TestRunComponentEnabler componentEnabler;
	private GotoCondition currentCondition;
	
	public GotoStepDesktopPanel(WsdlGotoTestStep testStep)
	{
		super( new BorderLayout() );
		this.gotoStep = testStep;
		componentEnabler = new TestRunComponentEnabler( testStep.getTestCase() );
		
		buildUI();
	}
	
	public TestRunComponentEnabler getComponentEnabler()
	{
		return componentEnabler;
	}

	public JList getConditionList()
	{
		return conditionList;
	}

	public JButton getCopyButton()
	{
		return copyButton;
	}

	public JButton getDeclareButton()
	{
		return declareButton;
	}

	public JButton getDeleteButton()
	{
		return deleteButton;
	}

	public JTextArea getExpressionArea()
	{
		return expressionArea;
	}

	public WsdlGotoTestStep getGotoStep()
	{
		return gotoStep;
	}

	public DefaultListModel getListModel()
	{
		return listModel;
	}

	public JButton getTestConditionButton()
	{
		return testConditionButton;
	}

	public JComboBox getTestStepsCombo()
	{
		return testStepsCombo;
	}

	public GotoTestStepsComboBoxModel getTestStepsModel()
	{
		return testStepsModel;
	}

	private void buildUI()
	{
		JSplitPane splitPane = UISupport.createHorizontalSplit();
		
		listModel = new DefaultListModel();
		
		for( int c = 0; c < gotoStep.getConditionCount(); c++ )
		{
			listModel.addElement( gotoStep.getConditionAt( c  ).getName() );
		}
		
		conditionList = new JList( listModel );
		conditionList.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		conditionList.addListSelectionListener( new ConditionListSelectionListener());
		
		JScrollPane listScrollPane = new JScrollPane( conditionList );
		listScrollPane.setBorder( 
				BorderFactory.createCompoundBorder(
				BorderFactory.createTitledBorder( BorderFactory.createEmptyBorder(), "Conditions" ), 
				BorderFactory.createLineBorder( Color.GRAY )));
		
		splitPane.setLeftComponent( listScrollPane);
		
		expressionArea = new JUndoableTextArea();
		expressionArea.setEnabled( false );
		expressionArea.getDocument().addDocumentListener( new SourceAreaDocumentListener());
		
		JPanel expressionPanel = new JPanel( new BorderLayout() );
		expressionPanel.add( new JScrollPane( expressionArea ), BorderLayout.CENTER );
		ButtonBarBuilder builder = new ButtonBarBuilder();
		builder.addFixed( new JLabel( "<html><b>Condition XPath Expression</b></html>"));
		builder.addGlue();
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		expressionPanel.add( builder.getPanel(), BorderLayout.NORTH );
		expressionPanel.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		
		builder = buildToolbar();
		expressionPanel.add( builder.getPanel(), BorderLayout.SOUTH );
		
		splitPane.setRightComponent( expressionPanel );
		splitPane.setResizeWeight( 0.1 );
		splitPane.setDividerLocation( 120 );
		
		add( splitPane, BorderLayout.CENTER );
		
		builder = new ButtonBarBuilder();
		JButton addButton = new JButton( new AddAction() );
		builder.addFixed( addButton );
		builder.addRelatedGap();
		copyButton = new JButton( new CopyAction() );
		copyButton.setEnabled( false );
		builder.addFixed( copyButton);
		builder.addRelatedGap();
		deleteButton = new JButton( new DeleteAction() );
		deleteButton.setEnabled( false );
		builder.addFixed( deleteButton);
		builder.addRelatedGap();
		declareButton = new JButton( new DeclareNamespacesAction() );
		declareButton.setEnabled( false );
		builder.addFixed( declareButton);
		builder.addRelatedGap();
		JButton runButton = new JButton( new RunAction() );
		builder.addFixed( runButton);
		
		builder.addGlue();
		builder.addFixed( new JButton( new CloseAction() ));
		builder.addRelatedGap();
		builder.addFixed( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.GOTOSTEPEDITOR_HELP_URL )));
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ) );
		
		add( builder.getPanel(), BorderLayout.SOUTH );
		
		setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		setPreferredSize( new Dimension( 550, 300 ));
		
		if( listModel.getSize() > 0 ) 
			conditionList.setSelectedIndex( 0 );
		
		componentEnabler.add( conditionList );
		componentEnabler.add( expressionArea );
		componentEnabler.add( testStepsCombo );
		componentEnabler.add( testConditionButton );
		componentEnabler.add( copyButton );
		componentEnabler.add( declareButton );
		componentEnabler.add( deleteButton );
		componentEnabler.add( addButton );
		componentEnabler.add( runButton );
	}

	protected ButtonBarBuilder buildToolbar()
	{
		ButtonBarBuilder builder;
		builder = new ButtonBarBuilder();
		testStepsModel = new GotoTestStepsComboBoxModel( gotoStep.getTestCase(), null );
		testStepsCombo = new JComboBox( testStepsModel );
		testStepsCombo.setToolTipText( "The step the test case will go to if the current condition is true" );
		testStepsCombo.setEnabled( false );
		builder.addFixed( new JLabel( "<html><b>Target step:</b></html>"));
		builder.addRelatedGap();
		builder.addFixed( testStepsCombo );
		builder.addGlue();
		testConditionButton = new JButton( new TestConditionAction() );
		testConditionButton.setEnabled( false );
		builder.addFixed( testConditionButton);
		builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
		return builder;
	}
	
	private final class SourceAreaDocumentListener extends DocumentListenerAdapter
	{
		@Override
		public void update( Document document )
		{
			int ix = conditionList.getSelectedIndex();
			if( ix != -1 )
			{
				gotoStep.getConditionAt( ix ).setExpression( expressionArea.getText() );
			}
		}
	}
	
	private final class ConditionListSelectionListener implements ListSelectionListener
	{
		public void valueChanged(ListSelectionEvent e)
		{
			int ix = conditionList.getSelectedIndex();
			if( ix == -1 )
			{
				expressionArea.setText( "" );
				testStepsModel.setCondition( null );
				currentCondition = null;
			}
			else
			{
				currentCondition = gotoStep.getConditionAt( ix );
				expressionArea.setText( currentCondition.getExpression() );
				testStepsModel.setCondition( currentCondition );
			}

			boolean b = ix != -1;
			enableEditComponents( b );
		}
	}

	private final class AddAction extends AbstractAction
	{
		public AddAction()
		{
			super( "Add" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			String name = UISupport.prompt( "Specify name for condition", "Add Condition", "Condition " + 
					(gotoStep.getConditionCount()+1) );
			if( name == null || name.trim().length() == 0 ) return;
			
			gotoStep.addCondition( name );
			
			listModel.addElement( name );
			conditionList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class CopyAction extends AbstractAction
	{
		public CopyAction()
		{
			super( "Copy" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			int ix = conditionList.getSelectedIndex();
			GotoCondition config = gotoStep.getConditionAt( ix );
			
			String name = UISupport.prompt( "Specify name for condition", "Copy Condition", config.getName() );
			if( name == null || name.trim().length() == 0 ) return;
			
			GotoCondition condition = gotoStep.addCondition( name );
			condition.setExpression( config.getExpression() );
			condition.setTargetStep( config.getTargetStep() );
			condition.setType( config.getType() );
			
			listModel.addElement( name );
			conditionList.setSelectedIndex( listModel.getSize()-1 );
		}
	}
	
	private final class DeleteAction extends AbstractAction
	{
		public DeleteAction()
		{
			super( "Delete" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( UISupport.confirm( "Delete selected condition", "Delete Condition" )) 
			{
				int ix = conditionList.getSelectedIndex();
				
				conditionList.setSelectedIndex( -1 );
				
				gotoStep.removeConditionAt( ix );
				listModel.remove( ix );
				
				if( listModel.getSize() > 0 )
				{
					conditionList.setSelectedIndex( ix > listModel.getSize()-1 ? listModel.getSize()-1 : ix );
				}
			}
		}
	}
	
	private final class DeclareNamespacesAction extends AbstractAction
	{
		public DeclareNamespacesAction()
		{
			super( "Declare" );
			putValue( Action.SHORT_DESCRIPTION, "Declare available response namespaces in condition expression" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			try
			{
				WsdlTestRequestStep previousStep = (WsdlTestRequestStep) gotoStep.getTestCase().findPreviousStepOfType( 
						gotoStep, WsdlTestRequestStep.class );
				
				if (previousStep  != null )
				{
					WsdlResponse response = previousStep.getTestRequest().getResponse();
					String xml = response == null ? null : response.getContentAsString();
					if ( xml != null && xml.trim().length() > 0)
					{
						expressionArea.setText(XmlUtils.declareXPathNamespaces(xml)
								+ expressionArea.getText());
					}
					else
					{
						UISupport.showErrorMessage( "Missing response in previous request step [" + previousStep.getName() + "]" );
					}
				}
				else
				{
					UISupport.showErrorMessage( "Missing previous request step" );
				}
			}
			catch (Exception e1)
			{
				SoapUI.logError( e1 );
			}
		}
	}
	
	private final class RunAction extends AbstractAction
	{
		public RunAction()
		{
			super( "Run" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			if( listModel.getSize() == 0 )
			{
				UISupport.showErrorMessage( "Missing conditions!" );
				return;
			}

			WsdlTestRequestStep previousStep = (WsdlTestRequestStep) gotoStep.getTestCase().findPreviousStepOfType( 
					gotoStep, WsdlTestRequestStep.class );
			
			if( previousStep == null ) 
			{
				UISupport.showErrorMessage( "Missing previous request step" );
			}
			else
			{
				if( previousStep.getTestRequest().getResponse().getContentAsString() == null )
				{
					UISupport.showErrorMessage( "Missing response in previous message" );
					return;
				}
				
				GotoCondition target = gotoStep.runConditions( previousStep );
				if( target == null )
				{
					UISupport.showInfoMessage( "No condition true for current response in [" + previousStep.getName() + "]" );
				}
				else
				{
					UISupport.showInfoMessage( "Condition triggered for go to [" + target.getTargetStep() + "]" );
				}
			}
		}
	}
	
	private final class TestConditionAction extends AbstractAction
	{
		public TestConditionAction()
		{
			super( "Test Condition" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			WsdlTestRequestStep previousStep = (WsdlTestRequestStep) gotoStep.getTestCase().findPreviousStepOfType( 
					gotoStep, WsdlTestRequestStep.class );
			
			if( previousStep == null ) 
			{
				UISupport.showErrorMessage( "Missing previous request step" );
			}
			else
			{
				if( previousStep.getTestRequest().getResponse() == null || 
					 previousStep.getTestRequest().getResponse().getContentAsString().trim().length() == 0 )
				{
					UISupport.showErrorMessage( "Missing response in previous request step [" + 
							previousStep.getName() + "]" );
					return;
				}
				
				try
				{
					GotoCondition condition = gotoStep.getConditionAt( conditionList.getSelectedIndex() );
					boolean evaluate = condition.evaluate( previousStep );
					if( !evaluate )
					{
						UISupport.showInfoMessage( "Condition not true for current response in [" + previousStep.getName() + "]" );
					}
					else
					{
						UISupport.showInfoMessage( "Condition true for current response in [" + previousStep.getName() + "]" );
					}
				}
				catch (Exception e1)
				{
					UISupport.showErrorMessage( "Error checking condition: " + e1.getMessage() );
				}
			}
		}
	}
	
	private final class CloseAction extends AbstractAction
	{
		public CloseAction()
		{
			super( "Close" );
		}
		
		public void actionPerformed(ActionEvent e)
		{
			SoapUI.getDesktop().closeDesktopPanel( gotoStep );
		}
	}

	public WsdlGotoTestStep getModelItem()
	{
		return gotoStep;
	}

	public boolean onClose( boolean canCancel )
	{
		componentEnabler.release();
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}

	public boolean dependsOn(ModelItem modelItem)
	{
		return modelItem == gotoStep || modelItem == gotoStep.getTestCase() ||
				modelItem == gotoStep.getTestCase().getTestSuite() ||
				modelItem == gotoStep.getTestCase().getTestSuite().getProject();
	}

	public String getTitle()
	{
		return gotoStep.getTestCase().getName() + " - " + gotoStep.getName();
	}
	
	public String getDescription()
	{
		return "Goto: [" + gotoStep.getName() + "] - " + gotoStep.getTestStepTitle();
	}

	public Icon getIcon()
	{
		return getModelItem().getIcon();
	}

	public GotoCondition getCurrentCondition()
	{
		return currentCondition;
	}

	protected void enableEditComponents( boolean b )
	{
		expressionArea.setEnabled( b );
		testStepsCombo.setEnabled( b );
		copyButton.setEnabled( b );
		deleteButton.setEnabled( b );
		declareButton.setEnabled( b );
		testConditionButton.setEnabled( b );
	}
}
